      PROGRAM Noise_Poisson
 	    USE nrtype
      USE nr
      USE ran_state, ONLY : ran_seed
      IMPLICIT NONE

!  Declare arrays and some variables
      INTEGER(I4B), PARAMETER :: MAXN=20000000           ! maximum # data samples
      REAL(SP),DIMENSION(MAXN) :: T,X
      REAL(SP) ::  Factor,Rseed
      CHARACTER :: XFILE*50,DFILE*50,WFILE*50,HEADER*80
      INTEGER(I4B) :: I,J,K,N,Iseed
      INTEGER(I4B), DIMENSION(MAXN) :: IX

      write(*,*) "Add Poisson Noise (c) Vitaly Neustroev, Version 2013-Nov-14"
      write(*,*) 
      WRITE (*,*) 
      write(*,*) "INPUT FILE & PARAMETERS:"
      WRITE(*,100) ' Time-series data file: '
      READ (*,*) XFILE

      WRITE(*,100) ' Multiplier Factor: '
      READ (*,*) Factor
      Factor=abs(Factor)
      WRITE(*,100) ' Random Seed (Integer): '
      READ (*,*) Rseed     
      Iseed=int(Rseed)
      
      WRITE (*,*) 
      WRITE (*,*) '  Product will be converted to Integer'
      WRITE (*,*) 

!  Read the times T(1:N) and the data X(1:N) from XFILE
      N= MAXN                               ! defines max. N for RDDATA
      CALL RDDATA(N,T,X,XFILE)
      call ran_seed(sequence=3*Iseed)
      Do I=1,N
         IX(I)=INT(poidev(X(I)*Factor))
      EndDo
      CALL WRDATA(N,T,IX,'PoissonNoise.dat')
      CALL EXIT
100   Format(a,$)
      END


      SUBROUTINE RDDATA(N,TIME,DATA,DFILE)
!----------------------------------------------------------------------------
!  Author:  J. Lehar                                         Date:  13-JUL-87
!  Written for VAX FORTRAN 77   (May require some changes for other machines)
!  Source file:  READWRITE.FOR
!----------------------------------------------------------------------------
!  Reads in the times, data values for a time series from DFILE
!  In addition, a descriptive header is read in.  N (input) specifies the
!  maximum number of samples allowed and returns (output) the number of
!  samples found.  OPEN uses UNIT=1
!     ARGUMENTS:
!         N          (input) #samples allowed, (output) #samples found
!         TIME       (output) array for time samples
!         DATA       (output) array for data samples
!         DFILE      (input) data file name
!         HEADER     (output) descriptive file header
!----------------------------------------------------------------------------
  	    USE nrtype
        REAL(SP),DIMENSION(N) :: TIME,DATA
        CHARACTER DFILE*(*)

!  open the file & read in the header
        OPEN(UNIT=1,FILE=DFILE,STATUS='OLD')
!        READ(1,'(A80)') HEADER

!  read in the time series, each record has (Time,Data)
        I=1
!       step through the file until END-OF-FILE
!10         READ(1,'(2(1X,1P,E14.9))',END=11) TIME(I),DATA(I)
!10         READ(1,'(2(1X,E14.10))',END=11) TIME(I),DATA(I)
10          READ(1,*,END=11) TIME(I),DATA(I)
            I= I+1
            IF(I.GT.N)THEN
              TYPE *,'RDDATA: data exceeds array size, truncating at N=',N
              GOTO 11
           ENDIF
           GOTO 10
11      CONTINUE
        CLOSE(UNIT=1)
        N= I-1
        RETURN
        END


      SUBROUTINE WRDATA(N,TIME,X,DFILE)
!----------------------------------------------------------------------------
!  Author:  J. Lehar                                         Date:  13-JUL-87
!  Written for VAX FORTRAN 77   (May require some changes for other machines)
!  Source file:  READWRITE.FOR
!----------------------------------------------------------------------------
!  Writes the times, data values for a time series from DFILE
!  In addition, a descriptive header is written. OPEN uses UNIT=1
!     ARGUMENTS:
!         N          (input) #samples
!         TIME       (input) array for time samples
!         X          (input) array for data samples
!         DFILE      (input) data file name
!         HEADER     (input) descriptive file header
!----------------------------------------------------------------------------
  	    USE nrtype
        REAL(SP),DIMENSION(N) :: TIME        
        INTEGER(I4B),DIMENSION(N) :: X
        CHARACTER DFILE*(*)

!  open the file & write the header
        OPEN(UNIT=1,FILE=DFILE,STATUS='UNKNOWN')

!  write the time series
        DO I=1,N
           WRITE(1,*) TIME(I),X(I)
        ENDDO

!  close the file & return
        CLOSE(UNIT=1)
        RETURN
        END


